package logger

import (
	"fmt"
	"os/exec"
	"strings"

	"gopkg.in/yaml.v3"
)

type LoggerModule struct {
	settings Settings
	logger   Logger
}

type Settings struct {
	Name LoggerType `yaml:"name"`
}

type LoggerType string

const (
	LoggerTypeZap = "zap"
)

func (r LoggerType) Valid() bool {
	return r == LoggerTypeZap
}

func NewLoggerModule() *LoggerModule {
	return &LoggerModule{}
}

func (r *LoggerModule) Name() string {
	return "logger"
}

func (r *LoggerModule) SetSettings(settings any) error {
	out, err := yaml.Marshal(settings)
	if err != nil {
		return err
	}

	if err = yaml.Unmarshal(out, &r.settings); err != nil {
		return err
	}

	if !r.settings.Name.Valid() {
		return fmt.Errorf("invalid logger type")
	}

	switch r.settings.Name {
	case LoggerTypeZap:
		r.logger = NewZapLogger()
	}

	return nil
}

func (r *LoggerModule) Execute() error {
	value := strings.Replace(r.Import(), `"`, "", -1)
	cmd := exec.Command("go", "get", value)

	return cmd.Run()
}

type Logger interface {
	Import() string
	ImportCore() string
	Declaration() string
	Type() string
	Message(messageType string, message string, fields ...any) string
}

func (r *LoggerModule) Import() string {
	return r.logger.Import()
}

func (r *LoggerModule) ImportCore() string {
	return r.logger.ImportCore()
}

func (r *LoggerModule) Init() string {
	return r.logger.Declaration()
}

func (r *LoggerModule) Type() string {
	return r.logger.Type()
}

func (r *LoggerModule) Message(messageType, message string, fields ...any) string {
	return r.logger.Message(messageType, message, fields...)
}
