package logger

import (
	"fmt"

	"golang.org/x/text/cases"
	"golang.org/x/text/language"
)

type ZapLogger struct {
}

func NewZapLogger() *ZapLogger {
	return &ZapLogger{}
}

func (r *ZapLogger) Import() string {
	return `"go.uber.org/zap"`
}

func (r *ZapLogger) ImportCore() string {
	return `"go.uber.org/zap/zapcore"`
}

func (r *ZapLogger) Declaration() string {
	return `cfgLogger := zap.NewDevelopmentConfig()
	cfgLogger.EncoderConfig.EncodeLevel = zapcore.CapitalColorLevelEncoder
	cfgLogger.EncoderConfig.ConsoleSeparator = "    "

	logger, err := cfgLogger.Build()
	if err != nil {
		panic(err)
	}`
}

func (r *ZapLogger) Type() string {
	return "*zap.Logger"
}

func (r *ZapLogger) Message(messageType, message string, fields ...any) string {
	var fieldsToString string

	caser := cases.Title(language.English)
	messageType = caser.String(messageType)

	if !r.isAllowedMessageType(messageType) {
		return fmt.Sprintf(`//FIXME TEMPLATE ERROR: logger.Message messageType bad type. Allowed types: %v`, r.allowedMessageTypes())
	}

	if len(fields)%2 != 0 {
		return `//FIXME TEMPLATE ERROR: logger.Message need even number of fields. Example: logger.Message <messageType> <some msg> <fieldName> <fieldVal>...`
	}

	for i := 0; i < len(fields); i += 2 {
		fieldsToString += fmt.Sprintf(`zap.Any("%v", %v)`, fields[i], fields[i+1])

		if i+1 < len(fields)-1 {
			fieldsToString += ", "
		}
	}

	return fmt.Sprintf(`logger.%v("%v", %v)`, messageType, message, fieldsToString)
}

func (r *ZapLogger) isAllowedMessageType(messageType string) bool {
	for _, allowed := range r.allowedMessageTypes() {
		if messageType == allowed {
			return true
		}
	}

	return false
}

func (r *ZapLogger) allowedMessageTypes() []string {
	return []string{"DPanic", "Debug", "Error", "Fatal", "Info", "Panic", "Warn"}
}
