package logger

import "testing"

func TestZapLogger_InfoMsg(t *testing.T) {
	type args struct {
		messageType string
		message     string
		fields      []any
	}
	tests := []struct {
		name string
		r    *ZapLogger
		args args
		want string
	}{
		{
			name: "default",
			r:    &ZapLogger{},
			args: args{
				messageType: "info",
				message:     `some msg`,
				fields:      []any{`key1`, 1, `key2`, 2},
			},
			want: `logger.Info("some msg", zap.Any("key1", 1), zap.Any("key2", 2))`,
		},
		{
			name: "error need even number of fields",
			r:    &ZapLogger{},
			args: args{
				messageType: "info",
				message:     `some msg`,
				fields:      []any{`key1`},
			},
			want: `//FIXME TEMPLATE ERROR: logger.Message need even number of fields. Example: logger.Message <messageType> <some msg> <fieldName> <fieldVal>...`,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := tt.r.Message(tt.args.messageType, tt.args.message, tt.args.fields...); got != tt.want {
				t.Errorf("ZapLogger.Message() = %v, want %v", got, tt.want)
			}
		})
	}
}
